#! /bin/sh

#set -e
ARCH=i386
PKG=lt-2.2.2-mingw-mlib

libtoolPREFIX=/usr/local/libtool/2.2.2
automakePREFIX=/usr/local/automake/1.10.1
autoconfPREFIX=/usr/local/autoconf/2.61
aclocalVERSION=1.10


#===
PATH="$libtoolPREFIX/bin:$PATH"
PATH="$automakePREFIX/bin:$PATH"
PATH="$autoconfPREFIX/bin:$PATH"
. ../../mingw-env
export PATH

aclocal() {
"$automakePREFIX"/bin/aclocal \
  -I "$libtoolPREFIX"/share/aclocal \
  -I "$automakePREFIX"/share/aclocal-${aclocalVERSION} \
  -I "$mingwPREFIX"/share/aclocal \
  -I /usr/share/aclocal \
  ${1+"$@"}
}


#===
do_lib1() {
  test -d lib1 || mkdir lib1

  cat > lib1/foo.c <<EOF
#include <stdio.h>

extern void foo1(long n) {
  fprintf(stderr, "foo1(%ld)\n", n);
}
EOF

  cat > lib1/Makefile.am <<EOF
lib_LTLIBRARIES = libfoo1.la
libfoo1_la_SOURCES = foo.c
EOF
}


do_lib2() {
  test -d lib2 || mkdir lib2

  cat > lib2/foo.c <<EOF
#include <stdio.h>
extern void foo1(long n);

extern void foo2(long n) {
  foo1(n);
  fprintf(stderr, "foo2(%ld)\n", n);
}
EOF

  cat > lib2/Makefile.am <<EOF
lib_LTLIBRARIES = libfoo2.la
libfoo2_la_SOURCES = foo.c
libfoo2_la_LIBADD = ../lib1/libfoo1.la
EOF
}


do_appl() {
  test -d appl || mkdir appl

  cat > appl/foo.c <<EOF
#include <stdio.h>
#ifdef USE_SLEEP
#  include <windows.h>
#endif
extern void foo1(long n);
extern void foo2(long n);

int main() {
  foo1(11);
  foo2(22);
  fprintf(stderr, "main()\n");
#ifdef USE_SLEEP
  Sleep(10000); /*10 sec.*/
#endif
  return(126); /*to check exit code*/
}
EOF

  cat > appl/Makefile.am <<EOF
bin_PROGRAMS = foo
foo_SOURCES = foo.c
foo_LDADD = ../lib2/libfoo2.la
EOF
}


do_top() {
  test -d build-aux || mkdir build-aux

  cat > configure.ac <<EOF
AC_INIT([test], [0.0])
AC_PREREQ(2.60)
AC_CONFIG_AUX_DIR(build-aux)

AC_CANONICAL_HOST
AM_INIT_AUTOMAKE

AC_PROG_CC
LT_INIT([shared win32-dll])

AC_OUTPUT([
 lib1/Makefile
 lib2/Makefile
 appl/Makefile
 Makefile
])
EOF

  cat > Makefile.am <<EOF
SUBDIRS = lib1 lib2 appl
EOF
}


do_bootstrap() {
  do_lib1
  do_lib2
  do_appl
  do_top

  libtoolize --force --copy --automake
  aclocal
  automake --foreign --add-missing --copy
  autoconf
}


#===
#do_bootstrap
#./configure --prefix=/tmp/test/pkg/$PKG --build=${ARCH}-gnu-linux --host=${ARCH}-mingw32msvc
#make LDFLAGS='-no-undefined -avoid-version'

(
cd appl
DEBUGWRAPPER=-DDEBUGWRAPPER
${ARCH}-mingw32msvc-gcc -g -O2 $DEBUGWRAPPER -c -o .libs/lt-mdemo.o ../lt-mdemo.c
${ARCH}-mingw32msvc-gcc -g -O2 -o foo.exe .libs/lt-mdemo.o


# I get command prompt before program output.
# Exit code is alwais zero on wine 0.9.24 even if real program executed
# by execv from wrapper exit with non-zero.
# Wwork-around: execv->spawnvp
wine ./foo.exe
echo DONE=$?
sleep 2
./foo.exe #with binfmt
echo DONE=$?
ps axfu > /tmp/xx
)
